
package ONVIF::Device::Elements::SetNTP;
use strict;
use warnings;

{ # BLOCK to scope variables

sub get_xmlns { 'http://www.onvif.org/ver10/device/wsdl' }

__PACKAGE__->__set_name('SetNTP');
__PACKAGE__->__set_nillable();
__PACKAGE__->__set_minOccurs();
__PACKAGE__->__set_maxOccurs();
__PACKAGE__->__set_ref();

use base qw(
    SOAP::WSDL::XSD::Typelib::Element
    SOAP::WSDL::XSD::Typelib::ComplexType
);

our $XML_ATTRIBUTE_CLASS;
undef $XML_ATTRIBUTE_CLASS;

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}

use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables

my %FromDHCP_of :ATTR(:get<FromDHCP>);
my %NTPManual_of :ATTR(:get<NTPManual>);

__PACKAGE__->_factory(
    [ qw(        FromDHCP
        NTPManual

    ) ],
    {
        'FromDHCP' => \%FromDHCP_of,
        'NTPManual' => \%NTPManual_of,
    },
    {
        'FromDHCP' => 'SOAP::WSDL::XSD::Typelib::Builtin::boolean',
        'NTPManual' => 'ONVIF::Device::Types::NetworkHost',
    },
    {

        'FromDHCP' => 'FromDHCP',
        'NTPManual' => 'NTPManual',
    }
);

} # end BLOCK







} # end of BLOCK



1;


=pod

=head1 NAME

ONVIF::Device::Elements::SetNTP

=head1 DESCRIPTION

Perl data type class for the XML Schema defined element
SetNTP from the namespace http://www.onvif.org/ver10/device/wsdl.







=head1 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over

=item * FromDHCP

 $element->set_FromDHCP($data);
 $element->get_FromDHCP();




=item * NTPManual

 $element->set_NTPManual($data);
 $element->get_NTPManual();





=back


=head1 METHODS

=head2 new

 my $element = ONVIF::Device::Elements::SetNTP->new($data);

Constructor. The following data structure may be passed to new():

 {
   FromDHCP =>  $some_value, # boolean
   NTPManual =>  { # ONVIF::Device::Types::NetworkHost
     Type => $some_value, # NetworkHostType
     IPv4Address => $some_value, # IPv4Address
     IPv6Address => $some_value, # IPv6Address
     DNSname => $some_value, # DNSName
     Extension =>  { # ONVIF::Device::Types::NetworkHostExtension
     },
   },
 },

=head1 AUTHOR

Generated by SOAP::WSDL

=cut

