.swatch {
    border: 1px solid black;
    margin-left: 3px;
    padding: 0px;
}

.SourceOptions input,
.SourcePath input {
  width: 100%;
}

textarea,
input[name="newMonitor[Name]"],
input[name="newMonitor[Path]"],
input[name="newMonitor[SecondPath]"],
input[name="newMonitor[LabelFormat]"],
input[name="newMonitor[ControlDevice]"],
input[name="newMonitor[ControlAddress]"],
input[name="newMonitor[ONVIF_URL]"],
input[name="newMonitor[ONVIF_Events_Path]"],
input[name="newMonitor[ONVIF_Options]"],
input[name="newMonitor[ONVIF_Alarm_Text]"],
input[name="newMonitor[EventStartCommand]"],
input[name="newMonitor[EventEndCommand]"],
input[name="newMonitor[MQTT_Subscriptions]"] {
  width: 100%;
	min-width: 240px;
}
input[name="newMonitor[ONVIF_URL]"] {
	min-width: 340px;
}
input[name="newMonitor[ONVIF_Username]"],
input[name="newMonitor[ONVIF_Password]"],
input[name="newMonitor[LabelFormat]"]{
	min-width: 240px;
}
input[name="newMonitor[AnalysisFPSLimit]"],
input[name="newMonitor[MaxFPS]"],
input[name="newMonitor[AlarmMaxFPS]"],
input[name="newMonitor[V4LCapturesPerFrame]"]{
    width: 75px;
}
input[name="newMonitor[Port]"],
input[name="newMonitor[Refresh]"],
input[name="newMonitor[LabelX]"],
input[name="newMonitor[LabelY]"],
input[name="newMonitor[ImageBufferCount]"],
input[name="newMonitor[MaxImageBufferCount]"],
input[name="newMonitor[WarmupCount]"],
input[name="newMonitor[PreEventCount]"],
input[name="newMonitor[PostEventCount]"],
input[name="newMonitor[StreamReplayBuffer]"],
input[name="newMonitor[AlarmFrameCount]"],
input[name="newMonitor[AutoStopTimeout]"],
input[name="newMonitor[TrackDelay]"],
input[name="newMonitor[ReturnDelay]"],
input[name="newMonitor[Width]"],
input[name="newMonitor[Height]"] {
  width: 80px;
}
#page tr td:first-child {
  min-width: 300px;
  vertical-align: top;
  text-align: right;
}

tr td input[type="checkbox"],
tr td input[type="radio"] {
  margin: 6px 0;
}

.tab-pane td:first-child {
  padding-right: 1rem;
  text-align: right;
}

body.sticky #content {
  overflow-y: auto;
  height: 100%;
}
nav ul.nav {
  height: 100%;
  flex-direction: column;
  flex-wrap: nowrap;
}

ul.form > li {
  text-align: left;
  padding-left: 300px;
  margin: 10px 0;
}
ul.form > li > label:first-child {
  width: 290px;
  margin-left: -300px;
  text-align: right;
}
.EncoderParameters label {
  vertical-align: top;
}

@media screen and (max-width:767px) {
  #content {
    display: block;
  }
  #pills-tab {
    justify-content: center;
    align-items: center;
    flex-direction: row;
    flex-wrap: wrap;
  }
  ul.form > li {
    padding-left: 0;
    margin: 5px 0;
  }
  ul.form > li > label:first-child {
    width: 100%;
    margin-left: 0;
    text-align: left;
  }
  #page tr td:first-child {
    min-width: unset;
  }
  .tab-pane td:first-child {
    text-align: left;
  }
  textarea,
  input[name="newMonitor[ONVIF_URL]"],
  input[name="newMonitor[ONVIF_Username]"],
  input[name="newMonitor[ONVIF_Password]"],
  input[name="newMonitor[LabelFormat]"] {
    min-width: unset;
  }
}
