.archived {
    background-color: #ffc107;
}

#contentTable.major .colTime {
    white-space: nowrap;
}

#contentTable.major .colId, #contentTable.major .colDuration, #contentTable.major .colFrames, #contentTable.major .colAlarmFrames, #contentTable.major .colTotScore, #contentTable.major .colMaxScore, #contentTable.major .colAvgScore, #eventTable th, #eventTable td {
    text-align: center;
}

#contentTable.major td.colDuration {
    text-align: right;
    padding-right: 8px;
}

#header {
    display: flex;
    justify-content: space-between;
}

#header h2, #header a {
    line-height: 1.1;
    margin:5px 0 0 0;
}

#header #info, #header #pagination, #header #controls {
    display: flex;
    flex-direction: column;
}

#header #controls {
    align-items: flex-end;
}

#header #pagination {
    align-items: center;
}

/* Dirty hack to fix up/down buttons on pagination number input */
input[type="number"].form-control {
  padding: 5px;
}

#events {
  height: 100%;
  overflow: visible;
  justify-content: center;
}

#objdetectModal  .modal-dialog {
  width: 65%;
  max-width: 65%;
}

#objdetectModal img {
  width: 100%;
}

body.sticky .bootstrap-table {
  height: 100%;
  display: flex;
  flex-direction: column;
}

body.sticky .fixed-table-container {
  overflow-y: auto;
}

body.sticky #eventTable thead {
  position: sticky;
  top: 0;
  box-shadow: 0 0px 0, 0 -3px 0 #dfe4ea;
}

.term-value, .chosen-container{
  width: 100% !important;
}

.bs-bars {
  width: 100%;
}
#toolbar .row {
  width: 100%;
}
#toolbar .row >div {
  padding: 0;
}
