#header {
}

#header form{
  width: 100%;
}

.container-fluid {
/*  padding-left: 0;
  padding-right: 0;*/
}

#wrapperMonitor {
  width: 100%;
  overflow-x: auto;
}

#wrapperMonitor .buttons {
  display: inline-block;
}

#sidebar {
  min-width: 140px;
  padding-right: 1px;
}

#sidebar .nav {
  flex-wrap: nowrap;
  text-align: left;
  overflow: auto;
}

#menuControls {
  display: flex;
  align-items: center;
}

#menuControls div {
  margin: 0 0 0 1em;
}

.monitor {
  display: flex;
  flex-direction: column;
  margin: 0 auto;
}

.monitor .imageFeed {
  border: 2px solid #999999;
  padding: 0px;
  margin: 0px;
}

.monitor .imageFeed img,
.monitor .imageFeed video,
.monitor .imageFeed video-stream,
.monitor .imageFeed svg {
  border: none;
  /*display: inline-block;*/
  display: block;
  width: 100%;
}

.monitorStream {
  text-align: center;
  margin: 0 auto;
  display: grid;
}

.monitorStream .button_zoom,
.monitorStream .zoompan {
  grid-area: 1/1;
}

#monitorStatus {
  margin: 4px auto;
  text-align: center;
  display: inline-block;
}

.monitorStatus > .monitorState {
  display: inline;
}

#replayStatus {
    margin: 3px 0 2px;
    text-align: center;
  display: inline-block;
}

#replayStatus > span {
    padding: 0 4px;
}

#dvrControls {
    margin-top: 3px;
    margin-bottom: 2px;
    text-align: center;
}

#dvrControls input {
    padding: 10px 10px;
    width: 50px;
    margin: 0 3px;
    font-weight: 900;
}

#dvrControls input[disabled] {
    color: #aaaaaa;
}

#dvrControls input.active {
    border: 0;
    background-color: #2ecc71;
    color: #fff;
}

#dvrControls input.inactive {
    border: 0;
    background-color: #e67e22;
    color: #fff;
}

#dvrControls input.unavail {
    background-color: #ccc;
    border: 0;
    cursor: default;
}


#events {
    margin: 0 auto;
}

#eventList {
    width: 100%;
}

#eventList thead td {
    font-weight: bold;
}

#eventList th, #eventList td {
    text-align: center;
}

li {
    display: inline;
    list-style-type: none;
}

span.alarm {
    color: #DC143C;
    font-weight: bold;
}

span.alert {
    color: #FF8C00;
    font-weight: bold;
}

#eventList tr.recent {
    background-color: #B0E0E6;
}

#eventList tr.highlight {
    background-color: #DCDCDC;
}

.controlHeader {
  width: 100%;
}

#viewingFPS,
#captureFPS,
#analysisFPS
{
  display: inline-block;
}
#stateValue,
#viewingFPSValue,
#captureFPSValue,
#analysisFPSValue
{
  display: inline-block;
  min-width: 40px;
  text-align: right;
}
.monitorStream img,
.monitorStream video {
  margin: 0 auto;
  width: 100%;
  height: 100%;
}
.bootstrap-table {
  width: 100%;
}

body.sticky #content {
  overflow-y: auto;
}

button.btn.btn-view-watch, button.btn.btn-fullscreen, .ratioControl {
  display: none;
}

#sizeControl > span {
  white-space: nowrap;
  line-height: 36px;
}

@media screen and (max-width: 767px) {
  #sidebar {
    width: 100%;
    min-width: unset;
  }

  #sidebar .nav {
    flex-wrap: wrap;
    flex-direction: row;
  }

  #wrapperMonitor {
    overflow-x: hidden;
  }

  #ptzControls {
    width: 100%;
  }

  .d-flex.flex-row.justify-content-between {
    flex-wrap: wrap;
  }
}

/* +++ Support for old ZoomPan algorithm */
.form-check.control-use-old-zoom-pan {
  display: inline-block;
  line-height: 32px;
  white-space: nowrap;
  top: 8px;
}
#headerButtons .form-check.control-use-old-zoom-pan input.form-check-input {
  margin-left: -1.0rem;
}
/* --- Support for old ZoomPan algorithm */
