<?php
//
// ZoneMinder web TR Turkish language file, 15th October 2024, Rev.1.0
// Copyright (C) 2024 T. Koray Peksayar
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

// Simple String Replacements
$SLANG = array(
	'24BitColour'=> '24 bit renk',
	'32BitColour'=> '32 bit renk',
	'8BitGrey'=> '8 bit gri tonlamalı',
	'APIEnabled'=> 'API Etkin',
	'AddMonitorDisabled'=> 'Kullanıcınızın yeni bir monitör eklemesine izin verilmiyor',
	'AddNewControl'=> 'Yeni Kontrol Ekle',
	'AddNewMonitor'=> 'Eklemek',
	'AddNewServer'=> 'Yeni Sunucu Ekle',
	'AddNewStorage'=> 'Yeni Depolama Ekle',
	'AddNewUser'=> 'Yeni Kullanıcı Ekle',
	'AddNewZone'=> 'Yeni Bölge Ekle',
	'AlarmBrFrames'=> 'Alarm<br/> Çerçeveler',
	'AlarmFrame'=> 'Alarm Çerçevesi',
	'AlarmFrameCount'=> 'Alarm Çerçeve Sayısı',
	'AlarmLimits'=> 'Alarm Limitleri',
	'AlarmMaximumFPS'=> 'Alarm Maksimum FPS',
	'AlarmPx'=> 'Alarm Px',
	'AlarmRGBUnset'=> 'Bir alarm RGB rengi ayarlamalısınız',
	'AlarmRefImageBlendPct'=> 'Alarm Referans Görüntü Karışımı %ge',
	'AllTokensRevoked'=> 'Tüm Tokenlar İptal Edildi',
	'AnalysisFPS'=> 'Analiz FPS',
	'AnalysisUpdateDelay'=> 'Analiz Güncelleme Gecikmesi',
	'ApplyingStateChange'=> 'Durum Değişikliğini Uygulama',
	'ArchArchived'=> 'Sadece Arşivlendi',
	'ArchUnarchived'=> 'Sadece Arşivden Çıkarılmış',
	'AreaUnits'=> 'Alan (px/%)',
	'AttrAlarmFrames'=> 'Alarm Çerçeveleri',
	'AttrAlarmedZone'=> 'Alarm Bölgesi',
	'AttrArchiveStatus'=> 'Arşiv Durumu',
	'AttrAvgScore'=> 'Ort. Puan',
	'AttrCause'=> 'Neden',
	'AttrDiskBlocks'=> 'Disk Blokları',
	'AttrDiskPercent'=> 'Disk Yüzdesi',
	'AttrDiskSpace'=> 'Dosya Sistemi Disk Alanı',
	'AttrDuration'=> 'Süre',
	'AttrEndDate'=> 'Bitiş Tarihi',
	'AttrEndDateTime'=> 'Bitiş Tarihi/Saati',
	'AttrEndTime'=> 'Son Zaman',
	'AttrEndWeekday'=> 'Hafta Sonu',
	'AttrEventDiskSpace'=> 'Olay Disk Alanı',
	'AttrFilterServer'=> 'Sunucu Filtresi Çalışıyor',
	'AttrFrames'=> 'Çerçeveler',
	'AttrId'=> 'İD',
	'AttrMaxScore'=> 'Maksimum Puan',
	'AttrMonitorId'=> 'Monitör Kimliği',
	'AttrMonitorName'=> 'Monitör Adı',
	'AttrMonitorServer'=> 'Sunucu İzleyicisi Çalışıyor',
	'AttrName'=> 'İsim',
	'AttrNotes'=> 'Notlar',
	'AttrSecondaryStorageArea'=> 'İkincil Depolama Alanı',
	'AttrStartDate'=> 'Başlangıç Tarihi',
	'AttrStartDateTime'=> 'Başlangıç Tarihi/Saati',
	'AttrStartTime'=> 'Başlangıç Zamanı',
	'AttrStartWeekday'=> 'Hafta içi başla',
	'AttrStateId'=> 'Çalıştırma Durumu',
	'AttrStorageArea'=> 'Depolama Alanı',
	'AttrStorageServer'=> 'Sunucu Barındırma Depolama',
	'AttrSystemLoad'=> 'Sistem Yükü',
	'AttrTotalScore'=> 'Toplam Puan',
	'AutoStopTimeout'=> 'Otomatik Durdurma Zaman Aşımı',
	'AvgBrScore'=> 'Ort.<br/> Gol',
	'BackgroundFilter'=> 'Arkaplanda filtreyi çalıştır',
	'BadAlarmFrameCount'=> 'Alarm kare sayısı bir veya daha fazla tam sayı olmalıdır',
	'BadAlarmMaxFPS'=> 'Alarm Maksimum FPS pozitif bir tam sayı veya kayan nokta değeri olmalıdır',
	'BadAnalysisFPS'=> 'Analiz FPS\'si pozitif tam sayı veya kayan nokta değeri olmalıdır',
	'BadAnalysisUpdateDelay'=> 'Analiz güncelleme gecikmesi sıfır veya daha büyük bir tam sayıya ayarlanmalıdır',
	'BadChannel'=> 'Kanal sıfır veya daha büyük bir tam sayıya ayarlanmalıdır',
	'BadColours'=> 'Hedef renk geçerli bir değere ayarlanmalıdır',
	'BadDevice'=> 'Aygıt geçerli bir değere ayarlanmalıdır',
	'BadEncoderParameters'=> 'Encoder crf için en azından bir değer olmadan iyi çalışmaz. Lütfen yardıma bakın.',
	'BadFPSReportInterval'=> 'FPS rapor aralığı arabellek sayısı 0 veya daha büyük bir tam sayı olmalıdır',
	'BadFormat'=> 'Biçim geçerli bir değere ayarlanmalıdır',
	'BadFrameSkip'=> 'Kare atlama sayısı sıfır veya daha büyük bir tam sayı olmalıdır',
	'BadHeight'=> 'Yükseklik geçerli bir değere ayarlanmalıdır',
	'BadHost'=> 'Ana bilgisayar geçerli bir IP adresine veya ana bilgisayar adına ayarlanmalıdır, http:// eklemeyin',
	'BadImageBufferCount'=> 'Görüntü arabellek boyutu 2 veya daha fazla tam sayı olmalıdır',
	'BadLabelX'=> 'Etiket X koordinatı sıfır veya daha büyük bir tam sayıya ayarlanmalıdır',
	'BadLabelY'=> 'Etiket Y koordinatı sıfır veya daha büyük bir tam sayıya ayarlanmalıdır',
	'BadMaxFPS'=> 'Maksimum FPS pozitif bir tam sayı veya kayan nokta değeri olmalıdır',
	'BadMotionFrameSkip'=> 'Hareket Kare atlama sayısı sıfır veya daha büyük bir tam sayı olmalıdır',
	'BadNameChars'=> 'İsimler yalnızca alfanümerik karakterlere ek olarak boşluk, tire ve alt çizgi içerebilir',
	'BadNoSaveJPEGsOrVideoWriter'=> 'SaveJPEGs ve VideoWriter ikisi de devre dışı olarak ayarlandı. Hiçbir şey kaydedilmeyecek!',
	'BadPalette'=> 'Palet geçerli bir değere ayarlanmalıdır',
	'BadPassthrough'=> 'Kayıt -> Geçiş sadece ffmpeg tipi monitörlerde çalışır.',
	'BadPath'=> 'Kaynak -> Yol geçerli bir değere ayarlanmalıdır',
	'BadPathNotEncoded'=> 'Kaynak -> Yol geçerli bir değere ayarlanmalıdır. URL yüzde kodlanması gerekebilecek geçersiz karakterler !*\'()$ ,#[] tespit ettik.',
	'BadPort'=> 'Kaynak -> Port geçerli bir sayıya ayarlanmalıdır',
	'BadPostEventCount'=> 'Etkinlik sonrası görüntü sayısı sıfır veya daha büyük bir tam sayı olmalıdır',
	'BadPreEventCount'=> 'Etkinlik öncesi görüntü sayısı en az sıfır ve görüntü arabellek boyutundan az olmalıdır',
	'BadPreEventCountMaxImageBufferCount'=> 'Maksimum Görüntü Arabellek Sayısı, Olay Öncesi görüntü sayısından büyük olmalıdır, aksi takdirde karşılanamaz',
	'BadRefBlendPerc'=> 'Referans karışım yüzdesi pozitif bir tam sayı olmalıdır',
	'BadSectionLength'=> 'Bölüm uzunluğu 30 veya daha büyük bir tam sayı olmalıdır',
	'BadSignalCheckColour'=> 'Sinyal kontrol rengi geçerli bir RGB renk dizisi olmalıdır',
	'BadSourceType'=> 'Kaynak Türü "Web Sitesi" İşlevinin "İzleme" olarak ayarlanmasını gerektirir',
	'BadStreamReplayBuffer'=> 'Akış tekrar oynatma arabelleği sıfır veya daha büyük bir tam sayı olmalıdır',
	'BadWarmupCount'=> 'Isınma kareleri sıfır veya daha büyük bir tam sayı olmalıdır',
	'BadWebColour'=> 'Web rengi geçerli bir web renk dizesi olmalıdır',
	'BadWebSitePath'=> 'Lütfen http:// veya https:// önekini de içeren tam web sitesi URL\'sini girin.',
	'BadWidth'=> 'Genişlik geçerli bir değere ayarlanmalıdır',
	'BandwidthHead'=> 'Bant genişliği',
	'BlobPx'=> 'Leke Px',
	'BlobSizes'=> 'Blob Boyutları',
	'CSSDescription'=> 'Bu oturum için CSS\'yi değiştirin',
	'CanAutoFocus'=> 'Otomatik Odaklama Yapılabilir mi?',
	'CanAutoGain'=> 'Otomatik Kazanç Olabilir mi',
	'CanAutoIris'=> 'Otomatik İris',
	'CanAutoWhite'=> 'Otomatik Beyaz Denge.',
	'CanAutoZoom'=> 'Otomatik Yakınlaştırma Yapılabilir mi?',
	'CanFocus'=> 'Odaklanabilir',
	'CanFocusAbs'=> 'Mutlak Odaklanabilir',
	'CanFocusCon'=> 'Sürekli Odaklanabilir',
	'CanFocusRel'=> 'Odaklanma Göreceli Olabilir',
	'CanGain'=> 'Kazanabilir',
	'CanGainAbs'=> 'Mutlak Kazanılabilir',
	'CanGainCon'=> 'Sürekli Kazanç Sağlayabilir',
	'CanGainRel'=> 'Akraba Kazanabilir',
	'CanIris'=> 'Iris olabilir mi?',
	'CanIrisAbs'=> 'İris Mutlak Olabilir mi?',
	'CanIrisCon'=> 'İris Sürekli Olabilir mi?',
	'CanIrisRel'=> 'Iris Akraba Olabilir mi',
	'CanMove'=> 'Hareket edebilir',
	'CanMoveAbs'=> 'Mutlak Hareket Edebilir',
	'CanMoveCon'=> 'Sürekli Hareket Edebilir',
	'CanMoveDiag'=> 'Çapraz Hareket Edebilir',
	'CanMoveMap'=> 'Haritalanmış Hareket Edebilir',
	'CanMoveRel'=> 'Göreceli Hareket Edebilir',
	'CanPan'=> 'Tava olabilir',
	'CanReboot'=> 'Yeniden başlatılabilir',
	'CanReset'=> 'Sıfırlanabilir',
	'CanSetPresets'=> 'Ön Ayarlar Ayarlanabilir',
	'CanSleep'=> 'Uyuyabilir',
	'CanTilt'=> 'Eğilebilir',
	'CanWake'=> 'Uyanabilir',
	'CanWhite'=> 'Beyaz Dengesi Olabilir mi?',
	'CanWhiteAbs'=> 'Beyaz Bal. Mutlak',
	'CanWhiteBal'=> 'Beyaz Bal.',
	'CanWhiteCon'=> 'Can White Bal.Sürekli',
	'CanWhiteRel'=> 'Can Beyaz Bal. Göreceli',
	'CanZoom'=> 'Yakınlaştırma Yapılabilir mi?',
	'CanZoomAbs'=> 'Zoom Absolute Olabilir mi?',
	'CanZoomCon'=> 'Yakınlaştırma Sürekli Olabilir mi',
	'CanZoomRel'=> 'Yakınlaştırma Göreceli Olabilir mi',
	'CancelForcedAlarm'=> 'Zorunlu Alarmı İptal Et',
	'CaptureHeight'=> 'Yakalama Yüksekliği',
	'CaptureMethod'=> 'Yakalama Yöntemi',
	'CapturePalette'=> 'Yakalama Paleti',
	'CaptureResolution'=> 'Çözünürlüğü Yakala',
	'CaptureWidth'=> 'Yakalama Genişliği',
	'CheckMethod'=> 'Alarm Kontrol Yöntemi',
	'ChooseDetectedCamera'=> 'Algılanan Kamerayı Seçin',
	'ChooseDetectedProfile'=> 'Algılanan Profili Seçin',
	'ChooseFilter'=> 'Filtreyi Seçin',
	'ChooseLogFormat'=> 'Bir günlük biçimi seçin',
	'ChooseLogSelection'=> 'Bir günlük seçimi seçin',
	'ChoosePreset'=> 'Ön Ayarı Seçin',
	'CloneMonitor'=> 'Klon',
	'ConcurrentFilter'=> 'Filtreyi eş zamanlı olarak çalıştır',
	'ConfigOptions'=> 'Yapılandırma Seçenekleri',
	'ConfigType'=> 'Yapılandırma Türü',
	'ConfigURL'=> 'Yapılandırma Tabanı URL\'si',
	'ConfiguredFor'=> 'İçin yapılandırıldı',
	'ConfirmDeleteControl'=> 'Uyarı, bir kontrolün silinmesi, onu kullanan tüm monitörlerin kontrol edilemez hale gelmesine neden olacaktır.<br/><br/> Silmek istediğinize emin misiniz?',
	'ConfirmDeleteDevices'=> 'Seçili cihazları silmek istediğinizden emin misiniz?',
	'ConfirmDeleteEvents'=> 'Seçili etkinlikleri silmek istediğinizden emin misiniz?',
	'ConfirmDeleteLayout'=> 'Mevcut düzeni silmek istediğinizden emin misiniz?',
	'ConfirmDeleteTitle'=> 'Silme Onayı',
	'ConfirmPassword'=> 'Şifreyi Onayla',
	'ConfirmUnarchiveEvents'=> 'Seçili etkinlikleri arşivden çıkarmak istediğinizden emin misiniz?',
	'ConjAnd'=> 'Ve',
	'ConjOr'=> 'veya',
	'ContactAdmin'=> 'Ayrıntılı bilgi için lütfen yöneticinizle iletişime geçin.',
	'ControlAddress'=> 'Kontrol Adresi',
	'ControlCap'=> 'Kontrol Yeteneği',
	'ControlCaps'=> 'Kontrol Yetenekleri',
	'ControlDevice'=> 'Kontrol Cihazı',
	'ControlType'=> 'Kontrol Türü',
	'Controllable'=> 'Kontrol edilebilir',
	'CycleWatch'=> 'Bisiklet Saati',
	'DateTime'=> 'Tarih/Saat',
	'DefaultCodec'=> 'Olay Görünümü İçin Varsayılan Yöntem',
	'DefaultRate'=> 'Varsayılan Oranı',
	'DefaultScale'=> 'Varsayılan Ölçek',
	'DefaultView'=> 'Varsayılan Görünüm',
	'DeleteAndNext'=> 'Sil ve Sonraki',
	'DeleteAndPrev'=> 'Sil ve Önceki',
	'DeleteSavedFilter'=> 'Kaydedilen filtreyi sil',
	'DetectedCameras'=> 'Algılanan Kameralar',
	'DetectedProfiles'=> 'Algılanan Profiller',
	'DeviceChannel'=> 'Cihaz Kanalı',
	'DeviceFormat'=> 'Aygıt Biçimi',
	'DeviceNumber'=> 'Cihaz Numarası',
	'DevicePath'=> 'Aygıt Yolu',
	'DisableAlarms'=> 'Alarmları devre dışı bırak',
	'DoNativeMotionDetection'=> 'Yerel Hareket Algılama Yapın',
	'Donate'=> 'Lütfen Bağış Yapın',
	'DonateAlready'=> 'Hayır, zaten bağış yaptım',
	'DonateEnticement'=> 'ZoneMinder\'ı bir süredir kullanıyorsunuz ve umarım evinizin veya işyerinizin güvenliği için faydalı bir eklenti buluyorsunuzdur. ZoneMinder ücretsiz ve açık kaynaklı olmasına ve öyle kalmasına rağmen, geliştirmek ve desteklemek için para harcamanız gerekir. Gelecekteki geliştirmeleri ve yeni özellikleri desteklemek isterseniz lütfen bağış yapmayı düşünün. Bağış yapmak elbette isteğe bağlıdır ancak çok takdir edilir ve istediğiniz kadar çok veya az bağış yapabilirsiniz.<br/><br/> Bağış yapmak isterseniz lütfen aşağıdaki seçeneği seçin veya tarayıcınızda <a href="https://zoneminder.com/donate/" target="_blank">https://zoneminder.com/donate/</a> adresine gidin.<br/><br/> ZoneMinder\'ı kullandığınız için teşekkür ederiz. ZoneMinder deneyiminizi daha da iyi hale getirmek için destek veya öneriler almak amacıyla <a href="https://forums.zoneminder.com">ZoneMinder.com\'daki</a> forumları ziyaret etmeyi unutmayın.',
	'DonateRemindDay'=> 'Henüz değil, 1 gün sonra tekrar hatırlatacağım',
	'DonateRemindHour'=> 'Henüz değil, 1 saat sonra tekrar hatırlat',
	'DonateRemindMonth'=> 'Henüz değil, 1 ay sonra tekrar hatırlatırım',
	'DonateRemindNever'=> 'Hayır, bağış yapmak istemiyorum, asla hatırlatmayacağım',
	'DonateRemindWeek'=> 'Henüz değil, 1 hafta sonra tekrar hatırlatacağım',
	'DonateYes'=> 'Evet, şimdi bağış yapmak istiyorum',
	'DownloadVideo'=> 'Videoyu İndir',
	'DuplicateMonitorName'=> 'Yinelenen Monitör Adı',
	'DuplicateRTSPStreamName'=> 'RTSP Akış Adını Kopyala',
	'EditControl'=> 'Düzenleme Kontrolü',
	'EditLayout'=> 'Düzeni Düzenle',
	'EnableAlarms'=> 'Alarmları Etkinleştir',
	'EnterNewFilterName'=> 'Yeni filtre adını girin',
	'ErrorBrackets'=> 'Hata, lütfen eşit sayıda açılış ve kapanış parantezi kullandığınızı kontrol edin',
	'ErrorValidValue'=> 'Hata, lütfen tüm terimlerin geçerli bir değere sahip olduğundan emin olun',
	'Etc'=> 'vesaire',
	'EventFilter'=> 'Olay Filtresi',
	'EventId'=> 'Olay Kimliği',
	'EventName'=> 'Etkinlik Adı',
	'EventPrefix'=> 'Etkinlik Öneki',
	'EventsLoading'=> 'Etkinlikler yükleniyor',
	'Exif'=> 'EXIF verilerini görüntüye gömün',
	'ExistsInFileSystem'=> 'Dosya Sisteminde Mevcut',
	'ExportCompress'=> 'Sıkıştırmayı Kullan',
	'ExportDetails'=> 'Etkinlik Ayrıntılarını Dışa Aktar',
	'ExportFailed'=> 'İhracat Başarısız',
	'ExportFormat'=> 'Dosya Biçimini Dışa Aktar',
	'ExportFormatTar'=> 'Katran',
	'ExportFormatZip'=> 'Posta kodu',
	'ExportFrames'=> 'Çerçeve Ayrıntılarını Dışa Aktar',
	'ExportImageFiles'=> 'Resim Dosyalarını Dışa Aktar',
	'ExportLog'=> 'İhracat Günlüğü',
	'ExportMatches'=> 'Maçları Dışa Aktar',
	'ExportMiscFiles'=> 'Diğer Dosyaları Dışa Aktar (eğer mevcutsa)',
	'ExportOptions'=> 'İhracat Seçenekleri',
	'ExportSucceeded'=> 'İhracat Başarılı',
	'ExportVideoFiles'=> 'Video Dosyalarını Dışa Aktar (eğer mevcutsa)',
	'Exporting'=> 'İhracat',
	'FPS'=> 'kare/saniye',
	'FPSReportInterval'=> 'FPS Rapor Aralığı',
	'FastForward'=> 'Hızlı İleri',
	'FilterArchiveEvents'=> 'Tüm maçları arşivle',
	'FilterCopyEvents'=> 'Tüm eşleşmeleri kopyala',
	'FilterDeleteEvents'=> 'Tüm eşleşmeleri sil',
	'FilterEmailBody'=> 'E-posta Gövdesi',
	'FilterEmailEvents'=> 'Tüm maçların e-posta ayrıntılarını gönder',
	'FilterEmailSubject'=> 'E-posta Konusu',
	'FilterEmailTo'=> 'E-postaya',
	'FilterExecuteEvents'=> 'Tüm eşleşmelerde komutu yürüt',
	'FilterLockRows'=> 'Satırları Kilitle',
	'FilterLog'=> 'Filtre günlüğü',
	'FilterMessageEvents'=> 'Tüm maçların mesaj detayları',
	'FilterMoveEvents'=> 'Tüm eşleşmeleri taşı',
	'FilterPx'=> 'Px Filtresi',
	'FilterUnarchiveEvents'=> 'Tüm eşleşmeleri arşivden çıkar',
	'FilterUnset'=> 'Bir filtre genişliği ve yüksekliği belirtmelisiniz',
	'FilterUpdateDiskSpace'=> 'Kullanılan disk alanını güncelle',
	'FilterUploadEvents'=> 'Tüm maçları yükle',
	'FilterUser'=> 'Filtreyi çalıştırmak için kullanıcı',
	'FilterVideoEvents'=> 'Tüm maçlar için video oluştur',
	'FlippedHori'=> 'Yatay olarak çevrilmiş',
	'FlippedVert'=> 'Dikey olarak çevrilmiş',
	'ForceAlarm'=> 'Kuvvet Alarmı',
	'FrameId'=> 'Çerçeve Kimliği',
	'FrameRate'=> 'Kare Hızı',
	'FrameSkip'=> 'Kare Atlama',
	'GenerateDownload'=> 'İndir Oluştur',
	'GenerateVideo'=> 'Video Oluştur',
	'GeneratingVideo'=> 'Video Oluşturma',
	'GetCurrentLocation'=> 'Mevcut Konumu Al',
	'GoToZoneMinder'=> 'ZoneMinder.com\'a gidin',
	'HasFocusSpeed'=> 'Odaklanma Hızı Var',
	'HasGainSpeed'=> 'Hız Kazandı',
	'HasHomePreset'=> 'Ana Sayfa Ön Ayarı Var',
	'HasIrisSpeed'=> 'İris Hızı Var',
	'HasPanSpeed'=> 'Pan Hızı Var',
	'HasPresets'=> 'Ön Ayarlara Sahip',
	'HasTiltSpeed'=> 'Eğim Hızı Var',
	'HasTurboPan'=> 'Turbo Tava Var',
	'HasTurboTilt'=> 'Turbo Eğim Var',
	'HasWhiteSpeed'=> 'Beyaz Dengesi Vardır. Hız',
	'HasZoomSpeed'=> 'Yakınlaştırma Hızı Var',
	'Highband'=> 'Yüksek S/B',
	'ImageBufferSize'=> 'Görüntü Arabellek Boyutu (kare)',
	'InvalidateTokens'=> 'Oluşturulan tüm belirteçleri geçersiz kıl',
	'KeyString'=> 'Anahtar Dizisi',
	'LimitResultsPost'=> 'sadece sonuçlar',
	'LimitResultsPre'=> 'İlkine sınır koy',
	'LinkedMonitors'=> 'Bağlantılı Monitörler',
	'ListMatches'=> 'Liste Eşleşmeleri',
	'LoggedInAs'=> 'Giriş yapıldı',
	'LoggingIn'=> 'Giriş Yapılıyor',
	'Lowband'=> 'Düşük S/B',
	'MaxBandwidth'=> 'Maksimum Bant Genişliği',
	'MaxBrScore'=> 'Maksimum.<br/> Gol',
	'MaxFocusRange'=> 'Maksimum Odak Aralığı',
	'MaxFocusSpeed'=> 'Maksimum Odaklanma Hızı',
	'MaxFocusStep'=> 'Maksimum Odak Adımı',
	'MaxGainRange'=> 'Maksimum Kazanç Aralığı',
	'MaxGainSpeed'=> 'Maksimum Kazanç Hızı',
	'MaxGainStep'=> 'Maksimum Kazanç Adımı',
	'MaxImageBufferCount'=> 'Maksimum Görüntü Arabellek Boyutu (kare)',
	'MaxIrisRange'=> 'Max Iris Serisi',
	'MaxIrisSpeed'=> 'Maksimum İris Hızı',
	'MaxIrisStep'=> 'Max Iris Adımı',
	'MaxPanRange'=> 'Max Pan Aralığı',
	'MaxPanSpeed'=> 'Maksimum Pan Hızı',
	'MaxPanStep'=> 'Maksimum Pan Adımı',
	'MaxTiltRange'=> 'Maksimum Eğim Aralığı',
	'MaxTiltSpeed'=> 'Maksimum Eğim Hızı',
	'MaxTiltStep'=> 'Maksimum Eğim Adımı',
	'MaxWhiteRange'=> 'Max White Bal. Serisi',
	'MaxWhiteSpeed'=> 'Max Beyaz Denge Hızı',
	'MaxWhiteStep'=> 'Max White Bal. Adım',
	'MaxZoomRange'=> 'Maksimum Yakınlaştırma Aralığı',
	'MaxZoomSpeed'=> 'Maksimum Yakınlaştırma Hızı',
	'MaxZoomStep'=> 'Maksimum Yakınlaştırma Adımı',
	'MaximumFPS'=> 'Maksimum FPS',
	'Medband'=> 'Orta S/B',
	'MetaConfig'=> 'Meta Yapılandırma',
	'MinAlarmAreaLtMax'=> 'Minimum alarm alanı maksimumdan az olmalıdır',
	'MinAlarmAreaUnset'=> 'Minimum alarm piksel sayısını belirtmelisiniz',
	'MinBlobAreaLtMax'=> 'Minimum blob alanı maksimumdan az olmalıdır',
	'MinBlobAreaUnset'=> 'Minimum blob piksel sayısını belirtmelisiniz',
	'MinBlobLtMinFilter'=> 'Minimum blob alanı, minimum filtre alanından küçük veya ona eşit olmalıdır',
	'MinBlobsLtMax'=> 'Minimum blob\'lar maksimumdan az olmalıdır',
	'MinBlobsUnset'=> 'Minimum blob sayısını belirtmelisiniz',
	'MinFilterAreaLtMax'=> 'Minimum filtre alanı maksimumdan az olmalıdır',
	'MinFilterAreaUnset'=> 'Minimum filtre piksel sayısını belirtmelisiniz',
	'MinFilterLtMinAlarm'=> 'Minimum filtre alanı, minimum alarm alanından küçük veya ona eşit olmalıdır',
	'MinFocusRange'=> 'Min Odak Aralığı',
	'MinFocusSpeed'=> 'Min Odaklanma Hızı',
	'MinFocusStep'=> 'Min Odaklanma Adımı',
	'MinGainRange'=> 'Min Kazanç Aralığı',
	'MinGainSpeed'=> 'Min Kazanç Hızı',
	'MinGainStep'=> 'Min Kazanç Adımı',
	'MinIrisRange'=> 'Min Iris Aralığı',
	'MinIrisSpeed'=> 'Min İris Hızı',
	'MinIrisStep'=> 'Min Iris Adımı',
	'MinPanRange'=> 'Min Pan Aralığı',
	'MinPanSpeed'=> 'Min Pan Hızı',
	'MinPanStep'=> 'Min Pan Adımı',
	'MinPixelThresLtMax'=> 'Minimum piksel eşiği maksimumdan az olmalıdır',
	'MinPixelThresUnset'=> 'Minimum piksel eşiğini belirtmeniz gerekir',
	'MinSectionlength'=> 'Minimum bölüm uzunluğu',
	'MinTiltRange'=> 'Min Eğim Aralığı',
	'MinTiltSpeed'=> 'Min Eğim Hızı',
	'MinTiltStep'=> 'Min Eğim Adımı',
	'MinWhiteRange'=> 'Min Beyaz Bal. Aralığı',
	'MinWhiteSpeed'=> 'Min Beyaz Denge Hızı',
	'MinWhiteStep'=> 'Min Beyaz Denge Adımı',
	'MinZoomRange'=> 'Min Yakınlaştırma Aralığı',
	'MinZoomSpeed'=> 'Min Yakınlaştırma Hızı',
	'MinZoomStep'=> 'Min Yakınlaştırma Adımı',
	'ModectDuringPTZ'=> 'PTZ hareketi sırasında hareket algılama yapın',
	'MonitorIds'=> 'Monitör Kimlikleri',
	'MonitorPreset'=> 'Monitör Ön Ayarı',
	'MonitorPresetIntro'=> 'Aşağıdaki listeden uygun bir ön ayar seçin.<br/><br/> Lütfen bunun, geçerli monitör için daha önce yapılandırdığınız değerlerin üzerine yazabileceğini unutmayın.<br/><br/>',
	'MonitorProbe'=> 'Monitör Probu',
	'MonitorProbeIntro'=> 'Aşağıdaki liste tespit edilen analog ve ağ kameralarını ve bunların halihazırda kullanılıp kullanılmadığını veya seçime açık olup olmadığını gösterir.<br/><br/> Aşağıdaki listeden istediğiniz girişi seçin.<br/><br/> Lütfen tüm kameraların algılanamayabileceğini ve burada bir kamera seçmenin, geçerli monitör için daha önce yapılandırdığınız değerlerin üzerine yazabileceğini unutmayın.<br/><br/>',
	'MontageReview'=> 'Montaj İncelemesi',
	'MotionFrameSkip'=> 'Hareketli Kare Atlama',
	'Mtg2widgrd'=> '2-geniş ızgara',
	'Mtg3widgrd'=> '3-geniş ızgara',
	'Mtg3widgrx'=> '3-geniş ızgara, ölçeklenebilir, alarm durumunda büyütülür',
	'Mtg4widgrd'=> '4-geniş ızgara',
	'MtgDefault'=> 'Varsayılan',
	'MustBeGe'=> 'büyük veya eşit olmalıdır',
	'MustBeLe'=> 'küçük veya eşit olmalı',
	'MustConfirmPassword'=> 'Şifreyi onaylamanız gerekiyor',
	'MustSupplyPassword'=> 'Bir şifre sağlamanız gerekiyor',
	'MustSupplyUsername'=> 'Bir kullanıcı adı sağlamalısınız',
	'NewGroup'=> 'Yeni Grup',
	'NewLabel'=> 'Yeni Etiket',
	'NewPassword'=> 'Yeni Şifre',
	'NewState'=> 'Yeni Devlet',
	'NewUser'=> 'Yeni Kullanıcı',
	'NextMonitor'=> 'Sonraki Monitör',
	'NoDetectedCameras'=> 'Algılanan Kamera Yok',
	'NoDetectedProfiles'=> 'Algılanan Profil Yok',
	'NoFramesRecorded'=> 'Bu etkinlik için kaydedilmiş kare yok',
	'NoGroup'=> 'Grup yok',
	'NoSavedFilters'=> 'Kaydedilmiş Filtre Yok',
	'NoStatisticsRecorded'=> 'Bu etkinlik/çerçeve için kayıtlı istatistik yok',
	'NoneAvailable'=> 'Hiçbiri mevcut değil',
	'NumPresets'=> 'Num Ön Ayarları',
	'OnvifCredentialsIntro'=> 'Lütfen seçili kamera için kullanıcı adı ve şifreyi girin.<br/> Eğer kamera için bir kullanıcı oluşturulmamışsa burada verilen kullanıcı, verilen şifre ile oluşturulacaktır.<br/><br/>',
	'OnvifProbe'=> 'ONVIF',
	'OnvifProbeIntro'=> 'Aşağıdaki liste tespit edilen ONVIF kameraları ve bunların halihazırda kullanılıp kullanılmadığını veya seçilebilip seçilemeyeceğini gösterir.<br/><br/> Aşağıdaki listeden istediğiniz girişi seçin.<br/><br/> Lütfen tüm kameraların algılanamayabileceğini ve burada bir kamera seçmenin, geçerli monitör için daha önce yapılandırdığınız değerlerin üzerine yazabileceğini unutmayın.<br/><br/>',
	'OpEq'=> 'eşit',
	'OpGt'=> 'daha büyük',
	'OpGtEq'=> 'büyük veya eşit',
	'OpIn'=> 'set halinde',
	'OpIs'=> 'dır',
	'OpIsNot'=> 'değil',
	'OpLike'=> 'içerir',
	'OpLt'=> 'daha az',
	'OpLtEq'=> 'küçük veya eşit',
	'OpMatches'=> 'maçlar',
	'OpNe'=> 'eşit değil',
	'OpNotIn'=> 'sette yok',
	'OpNotLike'=> 'içermez',
	'OpNotMatches'=> 'eşleşmiyor',
	'OptionHelp'=> 'Seçenek Yardımı',
	'OptionRestartWarning'=> 'Bu değişiklikler sistem çalışırken tam olarak yürürlüğe girmeyebilir. Değişikliklerinizi yapmayı bitirdiğinizde lütfen ZoneMinder\'ı yeniden başlattığınızdan emin olun.',
	'OptionalEncoderParam'=> 'İsteğe bağlı Kodlayıcı Parametreleri',
	'Options'=> 'Seçenekler',
	'OrEnterNewName'=> 'veya yeni bir isim girin',
	'Order'=> 'Emir',
	'OverwriteExisting'=> 'Mevcut olanın üzerine yaz',
	'PanLeft'=> 'Sola kaydır',
	'PanRight'=> 'Sağa kaydır',
	'PanTilt'=> 'Pan/Eğme',
	'ParentGroup'=> 'Ebeveyn Grubu',
	'PasswordsDifferent'=> 'Yeni ve onaylanan şifreler farklıdır',
	'PathToApi'=> 'API\'ye Giden Yol',
	'PathToIndex'=> 'Dizin Yolu',
	'PathToZMS'=> 'ZMS\'ye Giden Yol',
	'PauseCycle'=> 'Duraklama Döngüsü',
	'PhoneBW'=> 'Telefon S/B',
	'Pid'=> 'PID',
	'PixelDiff'=> 'Piksel Farkı',
	'Pixels'=> 'pikseller',
	'PlayAll'=> 'Tümünü Oynat',
	'PlayCycle'=> 'Oyun Döngüsü',
	'PleaseWait'=> 'Lütfen bekleyin',
	'PostEventImageBuffer'=> 'Etkinlik Sonrası Görüntü Sayısı',
	'PreEventImageBuffer'=> 'Etkinlik Öncesi Görüntü Sayısı',
	'PreserveAspect'=> 'En Boy Oranını Koru',
	'PreviewRate'=> 'Önizleme Hızı',
	'PreviousMonitor'=> 'Önceki Monitör',
	'PrivacyAbout'=> 'Hakkında',
	'PrivacyAboutText'=> 'ZoneMinder, 2002\'den beri Linux platformları için en iyi ücretsiz ve açık kaynaklı Video Yönetim Sistemi (VMS) çözümüdür. ZoneMinder topluluk tarafından desteklenir ve projeye boş zamanlarını gönüllü olarak vermeyi seçenler tarafından yönetilir. ZoneMinder\'ı geliştirmenin en iyi yolu dahil olmaktır.',
	'PrivacyConclusionText'=> 'Kameralarınızdan herhangi bir görüntüye özgü veri <u>toplamıyoruz</u> . Kameralarınızın ne izlediğini bilmiyoruz. Bu veriler burada belirtilmeyen hiçbir amaç için satılmayacak veya kullanılmayacaktır. Kabul et\'e tıklayarak, ZoneMinder\'ı daha iyi bir ürün haline getirmemize yardımcı olmak için bu verileri bize göndermeyi kabul edersiniz. Reddet\'e tıklayarak, ZoneMinder\'ı ve tüm özelliklerini özgürce kullanmaya devam edebilirsiniz.',
	'PrivacyContact'=> 'Temas etmek',
	'PrivacyContactText'=> 'Gizlilik politikamızla ilgili sorularınız veya bilgilerinizin kaldırılması için lütfen bizimle <a href="https://zoneminder.com/contact/">buradan</a> iletişime geçin.<br/><br/> Destek için toplulukla etkileşime geçmenin üç temel yolu vardır:<ul><li> ZoneMinder <a href="https://forums.zoneminder.com/">kullanıcı forumu</a></li><li> ZoneMinder <a href="https://zoneminder-chat.herokuapp.com/">Slack kanalı</a></li><li> ZoneMinder <a href="https://github.com/ZoneMinder/zoneminder/issues">Github forumu</a></li></ul><p> Github forumumuz yalnızca hata bildirimi içindir. Diğer tüm soru ve yorumlarınız için lütfen kullanıcı forumumuzu veya slack kanalımızı kullanın.</p>',
	'PrivacyCookies'=> 'Çerezler',
	'PrivacyCookiesText'=> 'ZoneMinder sunucusuyla iletişim kurmak için bir web tarayıcısı veya mobil uygulama kullansanız da, ZoneMinder sunucusuyla bir oturumu benzersiz şekilde tanımlamak için istemcide bir ZMSESSID çerezi oluşturulur. ZmCSS ve zmSkin çerezleri, stil ve görünüm tercihlerinizi hatırlamak için oluşturulur.',
	'PrivacyMonitorList'=> 'Her monitörden aşağıdaki yapılandırma parametreleri toplanır:<ul><li> İD</li><li> İsim</li><li> Üretici</li><li> Örnek</li><li> Tip</li><li> İşlev</li><li> Genişlik</li><li> Yükseklik</li><li> Renkler</li><li> Maksimum FPS</li><li> AlarmMaxFPS</li></ul>',
	'PrivacyTelemetry'=> 'Telemetri',
	'PrivacyTelemetryList'=> 'ZoneMinder Telemetry daemon\'ı sisteminiz hakkında aşağıdaki verileri toplar:<ul><li> Benzersiz bir tanımlayıcı (UUID)</li><li> Şehir bazlı konum <a href="https://ipinfo.io/geo">ipinfo.io</a> sorgulanarak toplanır. Şehir, bölge, ülke, enlem ve boylam parametreleri kaydedilir. Enlem ve boylam koordinatları yalnızca şehir veya kasaba düzeyine kadar doğrudur!</li><li> Şu anki zaman</li><li> Toplam monitör sayısı</li><li> Toplam etkinlik sayısı</li><li> Sistem mimarisi</li><li> İşletim sistemi çekirdeği, dağıtım ve dağıtım sürümü</li><li> ZoneMinder\'ın sürümü</li><li> Toplam bellek miktarı</li><li> CPU çekirdek sayısı</li></ul>',
	'PrivacyTelemetryText'=> 'ZoneMinder açık kaynaklı olduğundan, herkes kayıt olmadan kurabilir. Bu, şu gibi soruların yanıtlanmasını zorlaştırır: kaç sistem var, en büyük sistem hangisi, ne tür sistemler var veya bu sistemler nerede bulunuyor? Bu soruların yanıtlarını bilmek, bize bu soruları soran kullanıcılara yardımcı olur ve çoğunluk kullanıcı tabanına göre öncelikleri belirlememize yardımcı olur.',
	'Probe'=> 'Sondaj',
	'ProfileProbe'=> 'Akış Sondası',
	'ProfileProbeIntro'=> 'Aşağıdaki liste seçili kameranın mevcut akış profillerini gösterir.<br/><br/> Aşağıdaki listeden istediğiniz girişi seçin.<br/><br/> ZoneMinder\'ın ek profiller yapılandıramayacağını ve burada bir kamera seçmenin, geçerli monitör için daha önce yapılandırdığınız değerlerin üzerine yazabileceğini lütfen unutmayın.<br/><br/>',
	'RTSPDescribe'=> 'RTSP Yanıt Medya URL\'sini kullanın',
	'RTSPTransport'=> 'RTSP Taşıma Protokolü',
	'RecaptchaWarning'=> 'reCaptcha gizli anahtarınız geçersiz. Lütfen düzeltin, aksi takdirde reCaptcha çalışmayacaktır',
	'RecordAudio'=> 'Bir etkinliği kaydederken ses akışının saklanıp saklanmayacağı.',
	'RefImageBlendPct'=> 'Referans Görüntüsü %ge karışımı',
	'RemoteHostName'=> 'Ana Bilgisayar Adı',
	'RemoteHostPath'=> 'Yol',
	'RemoteHostPort'=> 'Liman',
	'RemoteHostSubPath'=> 'Alt yol',
	'RemoteImageColours'=> 'Görüntü Renkleri',
	'RemoteMethod'=> 'Yöntem',
	'RemoteProtocol'=> 'Protokol',
	'ReplayAll'=> 'Tüm Etkinlikler',
	'ReplayGapless'=> 'Boşluksuz Etkinlikler',
	'ReplaySingle'=> 'Tek Etkinlik',
	'ReportEventAudit'=> 'Denetim Olayları Raporu',
	'ResetEventCounts'=> 'Olay Sayılarını Sıfırla',
	'RestrictedCameraIds'=> 'Sınırlandırılmış Kamera Kimlikleri',
	'RestrictedMonitors'=> 'Kısıtlı Monitörler',
	'ReturnDelay'=> 'İade Gecikmesi',
	'ReturnLocation'=> 'İade Yeri',
	'RevokeAllTokens'=> 'Tüm Jetonları İptal Et',
	'RotateLeft'=> 'Sola Döndür',
	'RotateRight'=> 'Sağa Döndür',
	'RunAudit'=> 'Denetim Sürecini Çalıştır',
	'RunEventNotification'=> 'Olay Bildirimi İşlemini Çalıştır',
	'RunLocalUpdate'=> 'Güncellemek için lütfen zmupdate.pl\'yi çalıştırın',
	'RunMode'=> 'Çalışma Modu',
	'RunState'=> 'Çalıştırma Durumu',
	'RunStats'=> 'İstatistik İşlemini Çalıştır',
	'RunTrigger'=> 'Tetikleyici İşlemi Çalıştır',
	'SaveAs'=> 'Farklı kaydet',
	'SaveFilter'=> 'Filtreyi Kaydet',
	'SaveJPEGs'=> 'JPEG\'leri kaydet',
	'Sectionlength'=> 'Bölüm uzunluğu',
	'SelectFormat'=> 'Biçimi Seçin',
	'SelectLog'=> 'Günlüğü Seç',
	'SelectMonitors'=> 'Monitörleri Seçin',
	'SelfIntersecting'=> 'Çokgen kenarları kesişmemelidir',
	'SetNewBandwidth'=> 'Yeni Bant Genişliğini Ayarla',
	'SetPreset'=> 'Ön Ayarı Ayarla',
	'ShowFilterWindow'=> 'Filtre Penceresini Göster',
	'ShowTimeline'=> 'Zaman Çizelgesini Göster',
	'SignalCheckColour'=> 'Sinyal Kontrol Rengi',
	'SignalCheckPoints'=> 'Sinyal Kontrol Noktaları',
	'SkinDescription'=> 'Bu oturum için görünümü değiştir',
	'SortAsc'=> 'Yükselen',
	'SortBy'=> 'Göre sırala',
	'SortDesc'=> 'Açıklama',
	'SourceColours'=> 'Kaynak Renkler',
	'SourcePath'=> 'Kaynak Yolu',
	'SourceType'=> 'Kaynak Türü',
	'SpeedHigh'=> 'Yüksek Hız',
	'SpeedLow'=> 'Düşük Hız',
	'SpeedMedium'=> 'Orta Hız',
	'SpeedTurbo'=> 'Turbo Hız',
	'StatusConnected'=> 'Yakalama',
	'StatusNotRunning'=> 'Koşmuyor',
	'StatusRunning'=> 'Yakalanmıyor',
	'StatusUnknown'=> 'Bilinmeyen',
	'StepBack'=> 'Geri adım at',
	'StepForward'=> 'İleri Adım',
	'StepLarge'=> 'Büyük Adım',
	'StepMedium'=> 'Orta Adım',
	'StepNone'=> 'Adım yok',
	'StepSmall'=> 'Küçük Adım',
	'StorageArea'=> 'Depolama Alanı',
	'StorageDoDelete'=> 'Silinir mi',
	'StorageScheme'=> 'Şema',
	'StreamReplayBuffer'=> 'Akış Tekrarı Görüntü Arabelleği',
	'SystemLog'=> 'Sistem Günlüğü',
	'TZUnset'=> 'Ayarlanmamış - php.ini\'deki değeri kullan',
	'TargetColorspace'=> 'Hedef renk alanı',
	'TimeDelta'=> 'Zaman Deltası',
	'TimeStamp'=> 'Zaman Damgası',
	'TimelineTip1'=> 'Anlık görüntüyü ve etkinlik ayrıntılarını görüntülemek için farenizi grafiğin üzerine getirin.',
	'TimelineTip2'=> 'Etkinliği görüntülemek için grafiğin renkli bölümlerine veya görsele tıklayın.',
	'TimelineTip3'=> 'Tıklamanıza bağlı olarak daha küçük bir zaman dilimini yakınlaştırmak için arka plana tıklayın.',
	'TimelineTip4'=> 'Aşağıdaki kontrolleri kullanarak zaman aralığında uzaklaştırma yapabilir veya ileri geri gidebilirsiniz.',
	'TimestampLabelFormat'=> 'Zaman Damgası Etiket Biçimi',
	'TimestampLabelSize'=> 'Yazı Tipi Boyutu',
	'TimestampLabelX'=> 'Zaman Damgası Etiketi X',
	'TimestampLabelY'=> 'Zaman Damgası Etiketi Y',
	'TooManyEventsForTimeline'=> 'Zaman Çizelgesi için çok fazla olay. Monitör sayısını azaltın veya Zaman Çizelgesi\'nin görünür aralığını azaltın',
	'TotalBrScore'=> 'Toplam<br/> Gol',
	'TrackDelay'=> 'Parça Gecikmesi',
	'TrackMotion'=> 'Hareketi İzle',
	'TurboPanSpeed'=> 'Turbo Tava Hızı',
	'TurboTiltSpeed'=> 'Turbo Eğim Hızı',
	'UpdateAvailable'=> 'ZoneMinder için bir güncelleme mevcut.',
	'UpdateNotNecessary'=> 'Güncellemeye gerek yok.',
	'UseFilter'=> 'Filtreyi Kullan',
	'UseFilterExprsPost'=> 'filtre ifadeleri',
	'UseFilterExprsPre'=> 'Kullanmak',
	'UsedPlugins'=> 'Kullanılan Eklentiler',
	'V4LCapturesPerFrame'=> 'Kare Başına Yakalama Sayısı',
	'V4LMultiBuffer'=> 'Çoklu arabelleğe alma',
	'VersionIgnore'=> 'Bu sürümü yoksay',
	'VersionRemindDay'=> '1 gün sonra tekrar hatırlat',
	'VersionRemindHour'=> '1 saat sonra tekrar hatırlat',
	'VersionRemindMonth'=> '1 ay sonra tekrar hatırlat',
	'VersionRemindNever'=> 'Yeni sürümleri hatırlatmayın',
	'VersionRemindWeek'=> '1 hafta sonra tekrar hatırlat',
	'VideoFormat'=> 'Video Format',
	'VideoGenFailed'=> 'Video Oluşturma Başarısız Oldu!',
	'VideoGenFiles'=> 'Mevcut Video Dosyaları',
	'VideoGenNoFiles'=> 'Hiçbir Video Dosyası Bulunamadı',
	'VideoGenParms'=> 'Video Üretim Parametreleri',
	'VideoGenSucceeded'=> 'Video Üretimi Başardı!',
	'VideoSize'=> 'Video Boyutu',
	'VideoWriter'=> 'Video Yazarı',
	'ViewAll'=> 'Tümünü Görüntüle',
	'ViewEvent'=> 'Etkinliği Görüntüle',
	'ViewMatches'=> 'Maçları Görüntüle',
	'ViewPaged'=> 'Sayfalanmış Görünüm',
	'WarmupFrames'=> 'Isınma Çerçeveleri',
	'WebColour'=> 'Web Rengi',
	'WebSiteUrl'=> 'Web sitesi URL\'si',
	'WhiteBalance'=> 'Beyaz Dengesi',
	'X10ActivationString'=> 'X10 Etkinleştirme Dizisi',
	'X10InputAlarmString'=> 'X10 Giriş Alarm Dizisi',
	'X10OutputAlarmString'=> 'X10 Çıkış Alarm Dizisi',
	'YouNoPerms'=> 'Bu kaynağa erişim izniniz yok.',
	'ZoneAlarmColour'=> 'Alarm Rengi (Kırmızı/Yeşil/Mavi)',
	'ZoneArea'=> 'Bölge Alanı',
	'ZoneExtendAlarmFrames'=> 'Alarm Çerçeve Sayısını Genişlet',
	'ZoneFilterSize'=> 'Filtre Genişliği/Yüksekliği (piksel)',
	'ZoneMinMaxAlarmArea'=> 'Min/Maks Alarm Alanı',
	'ZoneMinMaxBlobArea'=> 'Min/Maks Blob Alanı',
	'ZoneMinMaxBlobs'=> "Min/Maks Blob\'lar",
	'ZoneMinMaxFiltArea'=> 'Min/Maks Filtrelenmiş Alan',
	'ZoneMinMaxPixelThres'=> 'Min/Maks Piksel Eşiği (0-255)',
	'ZoneMinderLog'=> 'ZoneMinder Günlüğü',
	'ZoneOverloadFrames'=> 'Aşırı Yük Çerçevesi Sayımı Yoksay',
	'ZoomIn'=> 'Yakınlaştır',
	'ZoomOut'=> 'Uzaklaştır',
	//language names translation
	'cs_cz'=>'Çekçe',
	'de_de'=>'Almanca',
	'dk_dk'=>'Danimarkaca',
	'en_gb'=>'Birleşik Krallık İngilizcesi',
	'en_us'=>'ABD İngilizcesi',
	'es_CR'=>'İspanyolca KostaRika',
	'es_ar'=>'İspanyolca Arjantin',
	'es_es'=>'İspanyolca İspanya',
	'es_la'=>'İspanyolca Latamca',
	'et_ee'=>'Estonyalı',
	'fr_fr'=>'Fransızca',
	'he_il'=>'İbranice',
	'hu_hu'=>'Macarca',
	'it_it'=>'İtalyanca',
	'ja_jp'=>'Japonca',
	'nl_nl'=>'Flamanca',
	'pl_pl'=>'Lehçe',
	'pt_br'=>'Brezilya Portekizcesi',
	'ro_ro'=>'Romence',
	'ru_ru'=>'Rusça',
	'se_se'=>'Sami',
	'tr_tr'=>'Türkçe',
	'zh_cn'=>'Basitleştirilmiş Çince',
	'zh_tw'=>'Geleneksel Çince',
);

// Complex replacements with formatting and/or placements, must be passed through sprintf
$CLANG = array(
	'CurrentLogin'=> 'Mevcut giriş \'%1$s\'',
	'EventCount'=> '$s\'yi %2 %1 $s',
	'LastEvents'=> 'Son %1 $s %2 $s',
	'LatestRelease'=> 'Son sürüm v%1$s, artık v%2$s\'niz var.',
	'MonitorCount'=> '%1 $s %2 $s',
	'MonitorFunction'=> '%1 $s İşlevini İzle',
	'RunningRecentVer'=> 'ZoneMinder\'ın en son sürümü olan v%s\'yi çalıştırıyorsunuz.',
	'VersionMismatch'=> 'Sürüm uyuşmuyor, sistem sürümü %1 $s, veritabanı %2 $s.'
);

// The next section allows you to describe a series of word ending and counts used to
// generate the correctly conjugated forms of words depending on a count that is associated
// with that word.
// This intended to allow phrases such a '0 potatoes', '1 potato', '2 potatoes'etc to
// conjugate correctly with the associated count.
// In some languages such as English this is fairly simple and can be expressed by assigning
// a count with a singular or plural form of a word and then finding the nearest (lower) value.
// So '0'of something generally ends in 's', 1 of something is singular and has no extra
// ending and 2 or more is a plural and ends in 's'also. So to find the ending for '187'of
// something you would find the nearest lower count (2) and use that ending.
//
// So examples of this would be
// $zmVlangPotato = array( 0=>'Potatoes', 1=>'Potato', 2=>'Potatoes');
// $zmVlangSheep = array( 0=>'Sheep');
//
// where you can have as few or as many entries in the array as necessary
// If your language is similar in form to this then use the same format and choose the
// appropriate zmVlang function below.
// If however you have a language with a different format of plural endings then another
// approach is required . For instance in Russian the word endings change continuously
// depending on the last digit (or digits) of the numerator. In this case then zmVlang
// arrays could be written so that the array index just represents an arbitrary 'type'
// and the zmVlang function does the calculation about which version is appropriate.
//
// So an example in Russian might be (using English words, and made up endings as I
// don't know any Russian!!)
// 'Potato'=> array( 1=>'Potati', 2=>'Potaton', 3=>'Potaten'),
//
// and the zmVlang function decides that the first form is used for counts ending in
// 0, 5-9 or 11-19 and the second form when ending in 1 etc.
//

// Variable arrays expressing plurality, see the zmVlang description above
$VLANG = array(
    'Event'                => array( 0=>'Olaylar', 1=>'Olay', 2=>'Olaylar'),
    'Monitor'              => array( 0=>'İzleyiciler', 1=>'İzleyici', 2=>'İzleyiciler'),
);
// You will need to choose or write a function that can correlate the plurality string arrays
// with variable counts. This is used to conjugate the Vlang arrays above with a number passed
// in to generate the correct noun form.
//
// In languages such as English this is fairly simple
// Note this still has to be used with printf etc to get the right formatting
function zmVlang($langVarArray, $count) {
  krsort($langVarArray);
  foreach ($langVarArray as $key=>$value) {
    if (abs($count) >= $key) {
      return $value;
    }
  }
  ZM\Error('Değişken dil dizesi ilişkilendirilemiyor');
}

// This is an version that could be used in the Russian example above
// The rules are that the first word form is used if the count ends in
// 0, 5-9 or 11-19. The second form is used then the count ends in 1
// (not including 11 as above) and the third form is used when the
// count ends in 2-4, again excluding any values ending in 12-14.
//
// function zmVlang( $langVarArray, $count )
// {
//  $secondlastdigit = substr( $count, -2, 1 );
//  $lastdigit = substr( $count, -1, 1 );
//  // or
//  // $secondlastdigit = ($count/10)%10;
//  // $lastdigit = $count%10;
//
//  // Get rid of the special cases first, the teens
//  if ( $secondlastdigit == 1 && $lastdigit != 0 )
//  {
//      return( $langVarArray[1] );
//  }
//  switch ( $lastdigit )
//  {
//      case 0 :
//      case 5 :
//      case 6 :
//      case 7 :
//      case 8 :
//      case 9 :
//      {
//          return( $langVarArray[1] );
//          break;
//      }
//      case 1 :
//      {
//          return( $langVarArray[2] );
//          break;
//      }
//      case 2 :
//      case 3 :
//      case 4 :
//      {
//          return( $langVarArray[3] );
//          break;
//      }
//  }
//  die( 'Error, unable to correlate variable language string');
// }

// This is an example of how the function is used in the code which you can uncomment and
// use to test your custom function.
//$monitors = array();
//$monitors[] = 1; // Choose any number
//echo sprintf( $CLANG['MonitorCount'], count($monitors), zmVlang( $VLANG['VlangMonitor'], count($monitors) ) );

// In this section you can override the default prompt and help texts for the options area
// These overrides are in the form show below where the array key represents the option name minus the initial ZM_
// So for example, to override the help text for ZM_LANG_DEFAULT do
$OLANG = array(
	'OPTIONS_FFMPEG'=> array(
		'Help'=> 
		'Bu alandaki parametreler FFmpeg\'e iletilir. Birden fazla parametre ,~~ ile ayrılabilir. Örnekler (tırnak işareti girmeyin)~~~~ "allowed_media_types=video" Veri türünü cam\'den gelen isteğe (audio, video, data) ayarlayın~~~~ "reorder_queue_size=nnn" Yeniden sıralanmış paketlerin işlenmesi için tampona paket sayısını ayarlayın'
	),
 	
	'OPTIONS_ENCODER_PARAMETERS'=> array(
		'Help'=>
		'Kodlama kodeğine geçirilen parametreler. name=value, , veya yeni satır ile ayrılır.~~ Örneğin kaliteyi değiştirmek için crf seçeneğini kullanın. 1 en iyisidir, 51 en kötüsüdür 23 varsayılandır.~~ ~~ crf=23~~ ~~ Farklı davranışları desteklemek için movflags değerini değiştirmek isteyebilirsiniz. Bazı kişiler frag_keyframe seçeneği nedeniyle videoları görüntülemede sorun yaşıyor, ancak bu seçeneğin tamamlanmamış olayların görüntülenmesine izin vermesi gerekiyor. Daha fazla bilgi için [https://ffmpeg.org/ffmpeg-formats.html](https://ffmpeg.org/ffmpeg-formats.html) adresine bakın. ZoneMinder\'ın varsayılanı frag_keyframe,empty_moov\'dur~~'
	),
    
	'OPTIONS_DECODERHWACCELNAME'=> array(
		'Help'=> 
		'Bu, ffmpeg-hwaccel komut satırı seçeneğine eşdeğerdir. Intel grafik desteğiyle "vaapi" kullanın. NVIDIA cuda desteği için "cuda" kullanın. Desteği kontrol etmek için komut satırında ffmpeg -hwaccels çalıştırın.'
	),

	'OPTIONS_DECODERHWACCELDEVICE'=> array(
		'Help'=> 
		'Bu, ffmpeg-hwaccel_device komut satırı seçeneğine eşdeğerdir. Bunu yalnızca birden fazla GPU\'nuz varsa belirtmeniz gerekir. Intel VAAPI için tipik bir değer /dev/dri/renderD128 olacaktır.'
	),
	
    'OPTIONS_RTSPTrans'=> array(
      		'Help'=>
		'Bu, FFmpeg için RTSP Taşıma Protokolünü ayarlar.~~
      		TCP - Taşıma protokolü olarak TCP\'yi (RTSP kontrol kanalı içinde iç içe geçirme) kullanın.~~
		UDP - Taşıma protokolü olarak UDP\'yi kullanın. Daha yüksek çözünürlüklü kameralar UDP kullanırken bazı \'lekelenmeler\'görülürseTCP\'yi deneyin~~
		UDP Çoklu Yayın - Taşıma protokolü olarak UDP Çoklu Yayınını kullanın~~
		HTTP - Vekil sunucuları geçirmek için yararlı olan HTTP tünellemesini taşıma protokolü olarak kullanın.~~'
	),
	
	'OPTIONS_LIBVLC'=> array(
		'Help'=> 
		'Bu alandaki parametreler libVLC\'ye iletilir. 
		Birden fazla parametre ,~~ ile ayrılabilir
		Örnekler (tırnak işareti girmeyin)~~~~
			"--rtp-client-port=nnn" Rtp verileri için kullanılacak yerel portu ayarlayın~~~~
			"--verbose=2" libVLC\'nin ayrıntı düzeyini ayarlayın'
	),
	
	'OPTIONS_EXIF'=> array(
		'Help'=> 'Her JPEG karesine EXIF ​​verilerini yerleştirmek için bu seçeneği etkinleştirin.'
	),
	
	'OPTIONS_RTSPDESCRIBE'=> array(
		'Help'=>
		'Bazen, ilk RTSP el sıkışması sırasında kamera güncellenmiş bir medya URL\'si gönderecektir.
		ZoneMinder\'a bu URL\'yi kullanmasını söylemek için bu seçeneği etkinleştirin.
		Kameradan gelen değeri yoksaymak ve monitör yapılandırmasında girilen değeri kullanmak için bu seçeneği devre dışı bırakın~~~~
		Genellikle bu etkinleştirilmelidir. Ancak, kameranın kendi URL\'sini yanlış alabileceği durumlar vardır, örneğin kamera bir güvenlik duvarından geçerken.'
  ),
  
	'OPTIONS_MAXFPS'=> array(
		'Help'=> 
		'Bu alan yerel olmayan cihazlar için kullanıldığında belirli sınırlamalara sahiptir.~~
		Bu sınırlamalara uyulmaması canlı videoda gecikmeye, düzensiz kare atlamasına ve kaçırılan olaylara neden olur~~
		Akış IP kameraları için, kare hızını azaltmak için bu alanı kullanmayın.
		Bunun yerine kare hızını kamerada ayarlayın. Geçmişte kameranın kare hızından daha yüksek bir değer ayarlanması tavsiye ediliyordu ancak bu artık gerekli değil veya iyi bir fikir değil.
		Bazıları, çoğunlukla eski IP kameralar anlık görüntü modunu destekler. Bu durumda ZoneMinder kamerayı aktif olarak yeni
		görüntüler için yokluyor. Bu durumda, alanı kullanmak güvenlidir.'
	),
	
	'OPTIONS_ALARMMAXFPS'=> array(
		'Help'=>
		'Bu alan yerel olmayan aygıtlar için kullanıldığında belirli sınırlamalara sahiptir.~~
		Bu sınırlamalara uyulmaması canlı videoda gecikmeye, düzensiz kare atlamasına
		ve kaçırılan olaylara neden olur~
		Bu ayar, bu durum meydana gelirse Maksimum FPS değerini geçersiz kılmanıza olanak tanır. Maksimum FPS
		ayarında olduğu gibi, bunu boş bırakmak hiçbir sınır olmadığı anlamına gelir.'
	),
	
	'OPTIONS_LINKED_MONITORS'=> array(
		'Help'=>
		'Bu alan, sisteminizde bu monitör için tetikleyici görevi gören diğer monitörleri seçmenize olanak tanır. Yani mülkünüzün bir yönünü kapsayan bir kameranız varsa, o kamera hareket veya diğer olayları algıladığında tüm kameraların kayıt yapmasını zorlayabilirsiniz. Bağlantılı monitörleri seçmek için \'Seç\'e tıklayın.
		Bu özellik ile dairesel bağımlılıklar oluşturmamaya çok dikkat edin çünkü sonsuza kadar devam eden alarmlarınız olacak ve bu kesinlikle istediğiniz şey değildir! Monitörlerin bağlantısını kaldırmak için ctrl-tıklama yapabilirsiniz.'
	),
 
 	'OPTIONS_CAPTURING'=> array(
    'Help'=> 'Yakalama ne zaman yapılır:~~~~
Hiçbiri: Bir işlem çalıştırma, yakalama yapma. Eski İşleve eşdeğer == Hiçbiri~~~~
İsteğe bağlı: Bir zmc işlemi çalışacak, ancak kameraya bağlanmadan önce bir görüntüleyiciyi (canlı görüntü, küçük resim veya rstp sunucu bağlantısı) bekleyecektir.~~~~
Her zaman: Bir zmc işlemi çalışacak ve hemen bağlanacak ve bağlı kalacaktır.~~~~
',
  ),
  'OPTIONS_RTSPSERVER'=> array(
    'Help'=> '
    ZM, RTSP\'yi yeniden yayınlayabilen veya monitör akışını RTSP\'ye dönüştürmeyi deneyebilen kendi RTSP sunucusunu sağlar. Bu, tek bir kameradan birden fazla istemci çekmek yerine ZM Ana Bilgisayar makinelerinin kaynaklarını kullanmak istiyorsanız yararlıdır.~~~~
NOT:~~
Seçenekler > Ağ > MIN_RTSP_PORT yapılandırılabilir.
     ',
    ),
  'OPTIONS_RTSPSTREAMNAME'=> array(
     'Help'=> '
     RTSPServer etkinleştirilmişse, bu, erişilebilir olacağı uç nokta olacaktır.
Örneğin, bu monitör kimliği 6 ise, MIN_RTSP_PORT=20000 ve RTSPServerName
"my_camera" olarak ayarlanmışsa, akışa rtsp://ZM_HOST:20006/my_camera adresinden erişin
     ',
    ),
  'FUNCTION_ANALYSIS_ENABLED'=> array(
    'Help'=> '
      Yakalanan videoda hareket algılamanın ne zaman gerçekleştirileceği.
Bu ayar, işlem başladığında varsayılan durumu ayarlar.
Daha sonra harici tetikleyiciler zmtrigger zmu veya web kullanıcı arayüzü aracılığıyla açılıp kapatılabilir.
Etkinleştirilmediğinde hareket algılama veya bağlı monitör denetimi gerçekleştirilmeyecek ve
hiçbir olay oluşturulmayacaktır.
      '
  ),
  'FUNCTION_DECODING'=> array(
    'Help'=> '
      Hareket algılama gerçekleştirilmediğinde ve hiçbir jpeg kaydedilmeden H264Passthrough kullanıldığında, isteğe bağlı olarak H264/H265 paketlerini kod çözmemeyi seçebiliriz. Bu, CPU kullanımını önemli ölçüde azaltacaktır.~~~~
Her zaman: her kare kod çözülecek, canlı görüntü ve küçük resimler mevcut olacak.~~~~
İsteğe Bağlı: yalnızca biri izlerken kod çözme yapın.~~~~
Anahtar Kareler: Yalnızca anahtar kareler kod çözülecek, bu nedenle görüntüleme kare hızı, kamerada ayarlanan anahtar kare aralığına bağlı olarak çok düşük olacaktır.~~~~
Hiçbiri: Hiçbir kare kod çözülmeyecek, canlı görüntü ve küçük resimler mevcut olmayacak~~~~
'
  ),
  'FUNCTION_RTSP2WEB_ENABLED'=> array(
    'Help'=> '
      RTSP2Web akış sunucusunu h264/h265 canlı görüntü için kullanma girişimi. Deneysel, ancak
önemli ölçüde daha iyi performansa olanak tanır.'
  ),
  'FUNCTION_RTSP2WEB_TYPE'=> array(
    'Help'=> '
      RTSP2Web, MSE\'yi (Medya Kaynağı Uzantıları), HLS\'yi (HTTP Canlı Yayını) ve WebRTC\'yi destekler.
Her birinin kendine göre avantajları vardır, WebRTC muhtemelen en performanslı olanıdır, ancak aynı zamanda kodekler konusunda en seçici olanıdır.'
  ),
  'FUNCTION_JANUS_ENABLED'=> array(
    'Help'=> '
      Janus akış sunucusunu h264/h265 canlı görüntü için kullanmayı deneyin. Deneysel, ancak önemli ölçüde daha iyi performansa olanak tanır.'
  ),
  'FUNCTION_JANUS_AUDIO_ENABLED'=> array(
    'Help'=> '
      Janus akışında sesi etkinleştirmeyi deneyin. Ses desteği olmayan kameralar için bir etkisi yoktur,
ancak kameranız tarayıcı tarafından desteklenmeyen bir ses biçimi gönderirse akışın oynatılmasını engelleyebilir.'
  ),
  'FUNCTION_JANUS_PROFILE_OVERRIDE'=> array(
    'Help'=> '
      Bir tarayıcının belirli bir akışı oynatmayı denemesini sağlayabilecek bir Profil Kimliğini manuel olarak ayarlayın. Evrensel olarak desteklenen bir değer için "42e01f"
değerini deneyin veya kaynak tarafından belirtilen Profil Kimliğini kullanmak için bunu boş bırakın.'
  ),
  'FUNCTION_RESTREAM'=> array(
    'Help'=> '
      Akış hizmetleri (Janus, Go2RTC, RTSP2Web) için kaynak olarak kamera yerine ZoneMinder RTSP yeniden akışını
kullanmak için bunu etkinleştirin. Kameranız doğrudan bir akış hizmetiyle çalışmıyorsa yararlıdır.'
  ),
  'FUNCTION_JANUS_RTSP_SESSION_TIMEOUT'=> array(
    'Help'=> '
    RTSP oturumu için saniye cinsinden bir zaman aşımı süresini geçersiz kılın veya ayarlayın. Janus günlüklerinde çok sayıda 401 yetkisiz yanıt görüyorsanız yararlıdır. Kaynaktan zaman aşımını (gönderildiyse) kullanmak için 0 olarak ayarlayın.'
  ),
  'ImageBufferCount'=> array(
    'Help'=> '
    /dev/shm \'de mevcut ham görüntü sayısı. Şu anda 3-5 aralığında ayarlanmalıdır. Canlı görüntüleme için kullanılır.'
  ),
  'MaxImageBufferCount'=> array(
    'Help'=> '
    Paket kuyruğunda tutulacak maksimum video paketi sayısı.
Paket kuyruğu normalde kendini yönetir ve geçiş modu kullanılıyorsa Ön Olay Sayısı karelerini veya son anahtar kareden bu yana olan tüm kareleri tutar. Monitörün çok fazla RAM tüketmesini önlemek için bir maksimum ayarlayabilirsiniz, ancak anahtar kare aralığınız bu değerden büyükse, etkinlikleriniz olması gereken tüm karelere sahip olmayabilir.
Bu konuda günlüklerinizde hatalar alırsınız. Bu nedenle anahtar kare aralığınızın düşük olduğundan veya yeterli RAM\'iniz olduğundan emin olun.
  '
  ),
// Help for soap_wsa issue with chinesse cameras
   'OPTIONS_SOAP_wsa'=> array(
    'Help'=> '
     Bir hata alırsanız devre dışı bırakın ~~~ Yenileme yapılamadı Hata 12 ActionNotSupported
<env:Text>Cihaz bu özelliği DESTEKLEMEZ</env:Text> ~~~ ONVIF\'i etkinleştirmeye/kullanmaya çalışırken ~~bu
çalışmasını sağlamaya yardımcı olabilir... ONVIF\'i tamamen uygulamayan bazı Çin kameralarında çalıştığı doğrulandı
    '
   ),

//    'LANG_DEFAULT'=> array(
//        'Prompt'=> "This is a new prompt for this option",
//        'Help'=> "This is some new help for this option which will be displayed in the window when the ? is clicked"
//    ),
);
