
package ONVIF::Device::Elements::SetDot1XConfiguration;
use strict;
use warnings;

{ # BLOCK to scope variables

sub get_xmlns { 'http://www.onvif.org/ver10/device/wsdl' }

__PACKAGE__->__set_name('SetDot1XConfiguration');
__PACKAGE__->__set_nillable();
__PACKAGE__->__set_minOccurs();
__PACKAGE__->__set_maxOccurs();
__PACKAGE__->__set_ref();

use base qw(
    SOAP::WSDL::XSD::Typelib::Element
    SOAP::WSDL::XSD::Typelib::ComplexType
);

our $XML_ATTRIBUTE_CLASS;
undef $XML_ATTRIBUTE_CLASS;

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}

use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables

my %Dot1XConfiguration_of :ATTR(:get<Dot1XConfiguration>);

__PACKAGE__->_factory(
    [ qw(        Dot1XConfiguration

    ) ],
    {
        'Dot1XConfiguration' => \%Dot1XConfiguration_of,
    },
    {
        'Dot1XConfiguration' => 'ONVIF::Device::Types::Dot1XConfiguration',
    },
    {

        'Dot1XConfiguration' => 'Dot1XConfiguration',
    }
);

} # end BLOCK







} # end of BLOCK



1;


=pod

=head1 NAME

ONVIF::Device::Elements::SetDot1XConfiguration

=head1 DESCRIPTION

Perl data type class for the XML Schema defined element
SetDot1XConfiguration from the namespace http://www.onvif.org/ver10/device/wsdl.







=head1 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over

=item * Dot1XConfiguration

 $element->set_Dot1XConfiguration($data);
 $element->get_Dot1XConfiguration();





=back


=head1 METHODS

=head2 new

 my $element = ONVIF::Device::Elements::SetDot1XConfiguration->new($data);

Constructor. The following data structure may be passed to new():

 {
   Dot1XConfiguration =>  { # ONVIF::Device::Types::Dot1XConfiguration
     Dot1XConfigurationToken => $some_value, # ReferenceToken
     Identity =>  $some_value, # string
     AnonymousID =>  $some_value, # string
     EAPMethod =>  $some_value, # int
     CACertificateID =>  $some_value, # token
     EAPMethodConfiguration =>  { # ONVIF::Device::Types::EAPMethodConfiguration
       TLSConfiguration =>  { # ONVIF::Device::Types::TLSConfiguration
         CertificateID =>  $some_value, # token
       },
       Password =>  $some_value, # string
       Extension =>  { # ONVIF::Device::Types::EapMethodExtension
       },
     },
     Extension =>  { # ONVIF::Device::Types::Dot1XConfigurationExtension
     },
   },
 },

=head1 AUTHOR

Generated by SOAP::WSDL

=cut

