#contentTable .colSelect {
  text-align: center;
}

#contentTable .colSelect input {
}
#content form {
/*  height: 100%;
  overflow: auto; */
}

form {
  /* padding-top: 2rem; */
}

/* +++ temporarily, for the best, you probably need to connect options.css and review layers.*/
:root {
  --height-block-before-table: 62px; /* Distance from top border of block where table is located to top border of table itself */
  --indent-bootstrap-row: 15px; /* Adjusting bootstrap class "row" */
}

body.sticky #options {
  height: calc(100% - 50px); /* It's bad, but it's temporary */
  overflow-x: hidden;
  overflow-y: auto;
}

#options {
  width: 100%;
}

#optionsContainer {
  overflow-x: auto;
  overflow-y: hidden;
  padding-top: 15px;
}

body.sticky .wrapper-scroll-table {
  position: absolute;
  margin-right: calc(var(--indent-bootstrap-row) * -1);
  margin-left: calc(var(--indent-bootstrap-row) * -1);
  padding-right: var(--indent-bootstrap-row);
  padding-left: var(--indent-bootstrap-row);
  width: 100%;
  height: calc(100% - var(--height-block-before-table));
  overflow-x: hidden; 
  overflow-y: auto; 
}
/* --- */
