.colId,
.colName,
.colServer,
.colEvents,
.colFirstEvent,
.colLastEvent,
.colMinGap,
.colMaxGap,
.colFileMissing,
.colZeroSize {
  text-align: left;
}

#results {
  height: 100%;
  overflow-y: auto;
}
