package ONVIF::PTZ::Types::Capabilities;
use strict;
use warnings;


__PACKAGE__->_set_element_form_qualified(1);

sub get_xmlns { 'http://www.onvif.org/ver20/ptz/wsdl' };

our $XML_ATTRIBUTE_CLASS = 'ONVIF::PTZ::Types::Capabilities::_Capabilities::XmlAttr';

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}

use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables


__PACKAGE__->_factory(
    [ qw(
    ) ],
    {
    },
    {
    },
    {

    }
);

} # end BLOCK




package ONVIF::PTZ::Types::Capabilities::_Capabilities::XmlAttr;
use base qw(SOAP::WSDL::XSD::Typelib::AttributeSet);

{ # BLOCK to scope variables

my %EFlip_of :ATTR(:get<EFlip>);
my %Reverse_of :ATTR(:get<Reverse>);
my %GetCompatibleConfigurations_of :ATTR(:get<GetCompatibleConfigurations>);

__PACKAGE__->_factory(
    [ qw(
        EFlip
        Reverse
        GetCompatibleConfigurations
    ) ],
    {

        EFlip => \%EFlip_of,

        Reverse => \%Reverse_of,

        GetCompatibleConfigurations => \%GetCompatibleConfigurations_of,
    },
    {
        EFlip => 'SOAP::WSDL::XSD::Typelib::Builtin::boolean',
        Reverse => 'SOAP::WSDL::XSD::Typelib::Builtin::boolean',
        GetCompatibleConfigurations => 'SOAP::WSDL::XSD::Typelib::Builtin::boolean',
    }
);

} # end BLOCK




1;


=pod

=head1 NAME

ONVIF::PTZ::Types::Capabilities

=head1 DESCRIPTION

Perl data type class for the XML Schema defined complexType
Capabilities from the namespace http://www.onvif.org/ver20/ptz/wsdl.






=head2 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over



=back


=head1 METHODS

=head2 new

Constructor. The following data structure may be passed to new():

 { # ONVIF::PTZ::Types::Capabilities
 },



=head2 attr

NOTE: Attribute documentation is experimental, and may be inaccurate.
See the correspondent WSDL/XML Schema if in question.

This class has additional attributes, accessibly via the C<attr()> method.

attr() returns an object of the class ONVIF::PTZ::Types::Capabilities::_Capabilities::XmlAttr.

The following attributes can be accessed on this object via the corresponding
get_/set_ methods:

=over

=item * EFlip

 Indicates whether or not EFlip is supported.



This attribute is of type L<SOAP::WSDL::XSD::Typelib::Builtin::boolean|SOAP::WSDL::XSD::Typelib::Builtin::boolean>.

=item * Reverse

 Indicates whether or not reversing of PT control direction is supported.



This attribute is of type L<SOAP::WSDL::XSD::Typelib::Builtin::boolean|SOAP::WSDL::XSD::Typelib::Builtin::boolean>.

=item * GetCompatibleConfigurations

 Indicates support for the GetCompatibleConfigurations command.



This attribute is of type L<SOAP::WSDL::XSD::Typelib::Builtin::boolean|SOAP::WSDL::XSD::Typelib::Builtin::boolean>.


=back




=head1 AUTHOR

Generated by SOAP::WSDL

=cut

